//###########################################################################//
//             		Convolution for GPU                      				 //
//                                                                           //
//              YI YANG, University of Central Florida                       //
//                            Feb. 2009                                      //
//###########################################################################//
// Disclaimer:                                                               //
// This code is provided on an "AS IS" basis, without warranty. The author   //
// does not have any liability to you or any other person or entity with     //
// respect to any liability, loss, or damage caused or alleged to have been  //
// caused directly or indirectly by this code.                               //
//###########################################################################//
// This project shoule be put in "$BROOK_DIR\samples\CPP\" directory.		 //
// The $BROOK_DIR means the directory where you install your brook.          //
// Execution file will be "$BROOK_DIR\samples\CPP\xp_x86_32\convolution.exe  //
// Run it like follow: convolution.exe -x 1024 -y 1024 -t -e				 //
//###########################################################################//
// This implementation uses float4 as input and output, and every thread     //
// computes 4 float4 output in the same column.                              //
//###########################################################################//


#include "convolution.h"

int
main(int argc, char** argv)
{
    Convolution<float> sm(argv[0]);
    int retVal = 0;

    // Parse command line options
    sm.ParseCommandLine(argc, argv);
    
    // Allocate buffers
    sm.allocateBuffer();

    // Fill the input buffers with random values
    sm.fillInputBuffer();
    
    // Run GPU kernel
    retVal = sm.run();

    // Display timing informations
    if(sm.info->Timing)
    {
        sm.printTimeInfo();
    }

    // Verify result with CPU
    if(sm.info->Verify)
    {
        sm.verifyResults();
    }

    // Compare performance with CPU
    if(sm.info->Performance)
    {
        sm.comparePerformance();
    }

    return retVal;
}
