//###########################################################################//
//             			Common File for GPU Program				             //
//                                                                           //
//              YI YANG, University of Central Florida                       //
//                            OCT. 2008                                      //
//###########################################################################//
// Disclaimer:                                                               //
// This code is provided on an "AS IS" basis, without warranty. The author   //
// does not have any liability to you or any other person or entity with     //
// respect to any liability, loss, or damage caused or alleged to have been  //
// caused directly or indirectly by this code.                               //
//###########################################################################//

#include <stdio.h>
#include <time.h>

#include <cutil.h>

#include "cudaCom.h"

// current timer
unsigned int timer = 0;

/**
 * print the interval time between two time call of this function.
 * When we first call this function, it will create timer.
 * The last call should set msg be null to destroy the timer.
 *! @param msg  : the msg of the interval time
 */
float timestamp(char* msg) {

	float f = 0;
	if (timer!=0) {	// destroy old timer and create new timer
	    // stop and destroy timer
	    CUT_SAFE_CALL(cutStopTimer(timer));

		//output the time
		f = cutGetTimerValue(timer);
//	    printf("Processing time [%s]: %f (ms)\n", msg?msg:"end", f);
	    CUT_SAFE_CALL(cutDeleteTimer(timer));
	}
	else { // first time
//	    printf("Start Timer\n");
	}

	// start timer
	if (msg) {
		CUT_SAFE_CALL(cutCreateTimer(&timer));
		CUT_SAFE_CALL(cutStartTimer(timer));
	}
	return f;

}


void usage(const char* name)
{
    fprintf(stderr,"Usage: %s [-e|-v|-q|-t|]"
            " [-w <integer>] [-h <integer>] "
            " [-x <integer>] [-y <integer>] "
            " [-r <Iteration Count>]\n\n", name);
    fprintf(stderr, "\t-?        Display this help information\n" );
    fprintf(stderr, "\t-e        Verify Computational Correctness\n" );
    fprintf(stderr, "\t-v        Display Verbose Output\n" );
    fprintf(stderr, "\t-q        Supress Display Output\n" );
    fprintf(stderr, "\t-t        Run the test\n" );
    fprintf(stderr, "\t-w  <int> Flag to specify input matrix width the data set is, default 256\n" );
    fprintf(stderr, "\t-h  <int> Flag to specify input matrix height the data set is, default 256\n" );
    fprintf(stderr, "\t-x  <int> Flag to specify kernel matrix width the data set is, default 256\n" );
    fprintf(stderr, "\t-y  <int> Flag to specify kernel matrix height the data set is, default 256\n" );
    fprintf(stderr, "\t-r  <int> Number of iterations to run the program\n" );
}

void parseCommandLine(int argc, char* argv[], CmdInfo* Info)
{
    Info->Verbose = 0;
    Info->Verify = 1;
    Info->Quiet = 1;
    Info->Test = 0;
    Info->Width = 256;
    Info->Height = 256;
    Info->KWidth = 7;
    Info->KHeight = 7;
    Info->Iterations = 1;

    for (int x = 1; x < argc; ++x)
    {
        switch(argv[x][1])
        {
            default:
                break;
            case 'q':
                Info->Quiet = 1;
                break;
            case '?':
                usage(argv[0]);
            case 'e':
                Info->Verify = 1;
                break;
            case 'v':
                Info->Verbose = 1;
                break;
            case 't':
                Info->Test = 1;
                break;
            case 'w':
                if (++x < argc)
                {
                    sscanf(argv[x], "%d", &Info->Width);
                }
                break;
            case 'h':
                if (++x < argc)
                {
                    sscanf(argv[x], "%d", &Info->Height);
                }
                break;
            case 'x':
                if (++x < argc)
                {
                    sscanf(argv[x], "%d", &Info->KWidth);
                }
                break;
            case 'y':
                if (++x < argc)
                {
                    sscanf(argv[x], "%d", &Info->KHeight);
                }
                break;
            case 'r':
                if (++x < argc)
                {
                    sscanf(argv[x], "%d", &Info->Iterations);
                }
                break;
        };
    }
}

